@@ -72,10 +72,10 @@ from dulwich.errors import (
     )
 from dulwich.pack import (
     Pack,
-    PackData,
     )
 from dulwich.repo import DictRefsContainer
 import os
+import select
 import tempfile
 import urllib
 import urlparse
@@ -184,6 +184,40 @@ class TCPGitSmartTransport(GitSmartTransport):
             thin_packs=thin_packs, report_activity=self._report_activity)
 
 
+class SSHSocketWrapper(object):
+
+    def __init__(self, sock):
+        self.sock = sock
+
+    def read(self, len=None):
+        return self.sock.recv(len)
+
+    def write(self, data):
+        return self.sock.write(data)
+
+    def can_read(self):
+        return len(select.select([self.sock.fileno()], [], [], 0)[0]) > 0
+
+
+class DulwichSSHVendor(dulwich.client.SSHVendor):
+
+    def __init__(self):
+        from bzrlib.transport import ssh
+        self.bzr_ssh_vendor = ssh._get_ssh_vendor()
+
+    def run_command(self, host, command, username=None, port=None):
+        connection = self.bzr_ssh_vendor.connect_ssh(username=username,
+            password=None, port=port, host=host, command=command)
+        (kind, io_object) = connection.get_sock_or_pipes()
+        if kind == 'socket':
+            return SSHSocketWrapper(io_object)
+        else:
+            raise AssertionError("Unknown io object kind %r'" % kind)
+
+
+#dulwich.client.get_ssh_vendor = DulwichSSHVendor
+
+
 class SSHGitSmartTransport(GitSmartTransport):
 
     _scheme = 'git+ssh'
@@ -230,7 +264,7 @@ class RemoteGitDir(GitDir):
                 trace.info("git: %s" % text)
         def wrap_determine_wants(refs_dict):
             return determine_wants(remote_refs_dict_to_container(refs_dict))
-        client = self._get_client(thin_packs=False)
+        client = self._get_client(thin_packs=True)
         try:
             refs_dict = client.fetch_pack(self._client_path, wrap_determine_wants,
                 graph_walker, pack_data, progress)
@@ -240,7 +274,7 @@ class RemoteGitDir(GitDir):
             raise parse_git_error(self.transport.external_url(), e)
 
     def send_pack(self, get_changed_refs, generate_pack_contents):
-        client = self._get_client(thin_packs=False)
+        client = self._get_client(thin_packs=True)
         try:
             return client.send_pack(self._client_path, get_changed_refs,
                 generate_pack_contents)
@@ -309,29 +343,21 @@ class EmptyObjectStoreIterator(dict):
 class TemporaryPackIterator(Pack):
 
     def __init__(self, path, resolve_ext_ref):
-        super(TemporaryPackIterator, self).__init__(path)
-        self.resolve_ext_ref = resolve_ext_ref
+        super(TemporaryPackIterator, self).__init__(
+            path, resolve_ext_ref=resolve_ext_ref)
+        self._idx_load = lambda: self._idx_load_or_generate(self._idx_path)
 
-    @property
-    def data(self):
-        if self._data is None:
-            self._data = PackData(self._data_path)
-        return self._data
-
-    @property
-    def index(self):
-        if self._idx is None:
-            if not os.path.exists(self._idx_path):
-                pb = ui.ui_factory.nested_progress_bar()
-                try:
-                    def report_progress(cur, total):
-                        pb.update("generating index", cur, total)
-                    self.data.create_index(self._idx_path, 
-                        progress=report_progress)
-                finally:
-                    pb.finished()
-            self._idx = load_pack_index(self._idx_path)
-        return self._idx
+    def _idx_load_or_generate(self, path):
+        if not os.path.exists(path):
+            pb = ui.ui_factory.nested_progress_bar()
+            try:
+                def report_progress(cur, total):
+                    pb.update("generating index", cur, total)
+                self.data.create_index(path,
+                    progress=report_progress)
+            finally:
+                pb.finished()
+        return load_pack_index(path)
 
     def __del__(self):
         if self._idx is not None:
@@ -384,7 +410,7 @@ class RemoteGitControlDirFormat(GitControlDirFormat):
             get_client = transport._get_client
             client_path = transport._get_path()
         elif urlparse.urlsplit(transport.external_url())[0] in ("http", "https"):
-            def get_client(thin_packs=False):
+            def get_client(thin_packs):
                 return BzrGitHttpClient(transport, thin_packs=thin_packs)
             client_path, _ = urlutils.split_segment_parameters(transport._path)
         else:
